using System;
using System.Collections;
using Server.Network;

namespace Server.Gumps
{
	public class WhoGump : Gump
	{
		public static void Initialize()
		{
			Commands.Register( "Admin", AccessLevel.Counselor, new CommandEventHandler( WhoList_OnCommand ) );
			Commands.Register( "Who", AccessLevel.Counselor, new CommandEventHandler( WhoList_OnCommand ) );
			Commands.Register( "WhoList", AccessLevel.Counselor, new CommandEventHandler( WhoList_OnCommand ) );
		}

		private static void WhoList_OnCommand( CommandEventArgs e )
		{
			e.Mobile.SendGump( new WhoGump( e.Mobile ) );
		}

		private Mobile m_Owner;
		private ArrayList m_Buttons;

		public WhoGump( Mobile owner ) : base( 0, 0 )
		{
			m_Owner = owner;
			m_Buttons = new ArrayList();

			AddPage( 0 );

			AddBackground( 0, 0, 260, 351, 5054 );

			AddImageTiled( 10, 10, 240, 23, 0x52 );
			AddImageTiled( 11, 11, 238, 21, 0xBBC );

			ArrayList states = NetState.Instances;

			int page = 1;
			int index = 0;

			for ( int i = 0; i < states.Count; ++i )
			{
				Mobile m = ((NetState)states[i]).Mobile;

				if ( m != null )
				{
					if ( index >= 14 )
					{
						AddButton( 230, 13, 0x15E1, 0x15E5, 0, GumpButtonType.Page, page + 1 );
						AddLabel( 198, 11, 0, "Next" );

						++page;
						index = 0;

						AddPage( page );

						AddButton( 13, 13, 0x15E3, 0x15E7, 0, GumpButtonType.Page, page - 1 );
						AddLabel( 33, 11, 0, "Previous" );
					}

					AddImageTiled( 10, 32 + (index * 22), 240, 23, 0x52 );
					AddImageTiled( 11, 33 + (index * 22), 238, 21, 0xBBC );
					AddLabelCropped( 13, 33 + (index * 22), 196, 21, GetHueFor( m ), m.Name );

					m_Buttons.Add( m );
					if ( owner != m )
					{
						AddButton( 230, 35 + (index * 22), 0x15E1, 0x15E5, m_Buttons.Count, GumpButtonType.Reply, 0 );
					}

					m_Buttons.Add( m );

					if ( owner.AccessLevel > m.AccessLevel )
					{
						AddButton( 214, 35 + (index * 22), 0x15E3, 0x15E7, m_Buttons.Count, GumpButtonType.Reply, 0 );
					}

					++index;
				}
			}
		}

		public override void OnResponse(NetState sender, RelayInfo info)
		{
			if ( info.ButtonID >= 1 && info.ButtonID <= m_Buttons.Count )
			{
				Mobile m = (Mobile)m_Buttons[info.ButtonID - 1];

				if ( m.NetState != null && m.Map != Map.Internal )
				{
					if ( (info.ButtonID % 2) == 1 ) // Go to
					{
						m_Owner.Map = m.Map;
						m_Owner.Location = m.Location;
					}
					else // Get
					{
						m.Map = m_Owner.Map;
						m.Location = m_Owner.Location;
					}
				}
				else
				{
					m_Owner.SendMessage( "They are not logged in." );
				}
			}
		}

		private static int GetHueFor( Mobile m )
		{
			switch ( m.AccessLevel )
			{
				case AccessLevel.Player:
				{
					if ( m.Kills >= 5 )
					{
						return 0x21;
					}
					else if ( m.Criminal )
					{
						return 0x3B1;
					}

					return 0x58;
				}
				case AccessLevel.Counselor: return 0x2;
				case AccessLevel.GameMaster: return 0x21;
				case AccessLevel.Seer: return 0x144;
				case AccessLevel.Administrator: return 0x516;
				default: return 0;
			}
		}
	}
}