// Craft System Script v.0.2 by SickLab

using System;
using Server.Items;

namespace Server.Scripts.CraftSystems
{
	public class CraftItem
	{
		private CraftResCol m_arCraftRes;
		private CraftSkillCol m_arCraftSkill;
		private Type m_Type;
		private string m_strGroup;
		private string m_strName;

		public CraftItem( Type type, string strGroup, string strName )
		{
			m_arCraftRes = new CraftResCol();
			m_arCraftSkill = new CraftSkillCol();

			m_Type = type;
			m_strGroup = strGroup;
			m_strName = strName;
		}

		public void AddRes( Type type, string name, int amount )
		{
			AddRes( type, name, amount, "" );
		}

		public void AddRes( Type type, string name, int amount, int localizedMessage )
		{
			CraftRes craftRes = new CraftRes( type, name, amount, localizedMessage );
			m_arCraftRes.Add( craftRes );
		}

		public void AddRes( Type type, string name, int amount, string strMessage )
		{
			CraftRes craftRes = new CraftRes( type, name, amount, strMessage );
			m_arCraftRes.Add( craftRes );
		}

		public void AddSkill( SkillName skillToMake, double minSkill, double maxSkill )
		{
			CraftSkill craftSkill = new CraftSkill( skillToMake, minSkill, maxSkill );
			m_arCraftSkill.Add( craftSkill );
		}

		public Type ItemType
		{
			get
			{
				return m_Type;
			}
		}

		public string ItemGroup
		{
			get
			{
				return m_strGroup;
			}
		}

		public string ItemName
		{
			get
			{
				return m_strName;
			}
		}

		public CraftResCol Ressources
		{
			get
			{
				return m_arCraftRes;
			}
		}

		public CraftSkillCol Skills
		{
			get
			{
				return m_arCraftSkill;
			}
		}

		public bool ConsumeRes( Mobile from )
		{
			Container ourPack = from.Backpack;

			if ( ourPack == null )
				return false;

			Type[] types = new Type[m_arCraftRes.Count];
			int[] amounts = new int[m_arCraftRes.Count];

			for ( int i = 0; i < types.Length; ++i )
			{
				CraftRes craftRes = m_arCraftRes.GetAt( i );

				types[i] = craftRes.ItemType;
				amounts[i] = craftRes.Amount;
			}

			int index = ourPack.ConsumeTotal( types, amounts );

			if ( index == -1 )
			{
				return true;
			}
			else
			{
				m_arCraftRes.GetAt( index ).SendMessage( from );
				return false;
			}
		}

		public bool SkillCheck( Mobile from )
		{
			bool bCheck = true;
			
			for (int i = 0;  i < m_arCraftSkill.Count; i++)
			{
				CraftSkill craftSkill = m_arCraftSkill.GetAt(i);

				if ( !from.CheckSkill( craftSkill.SkillToMake, craftSkill.MinSkill, craftSkill.MaxSkill ) )
				{
					// We do not return here.  Because we need to train all the other skill in the collection.
					bCheck = false;
				}
			}
		
			// If at least one check has failed, general check failed
			return bCheck;
		}

		public void PlayCraftEffect( Mobile from, CraftSystem craftSystem )
		{
			craftSystem.PlayCraftEffect( from );
		}

		public void Craft( Mobile from, CraftSystem craftSystem )
		{
			if ( from.BeginAction( typeof( CraftSystem ) ) )
			{
				if ( ConsumeRes( from ) )
				{
					int iMin = craftSystem.MinCraftEffect;
					int iMax = ( craftSystem.MaxCraftEffect - iMin ) + 1;
					int iRandom = Utility.Random( iMax );
					iRandom += iMin + 1;
					new InternalTimer( from, craftSystem, this, iRandom ).Start();
				}
				else
				{
					from.EndAction( typeof( CraftSystem ) );
				}
			}
			else
			{
				from.SendLocalizedMessage( 500119 ); // You must wait to perform another action
			}
		}

		// To Code
		public bool GiveBackMaterials( Mobile from )
		{
			// We have to remove the material before the making of the object,
			// and give it back to the mobile if the material are not lost, to
			// prevent the mobile from removing the items while he is crafting it.

			


			// Return true means that some material are lost.
			return true;
		}

		private class InternalTimer : Timer
		{
			private Mobile m_From;
			private int m_iCount;
			private int m_iCountMax;
			private CraftItem m_CraftItem;
			private CraftSystem m_CraftSystem;

			public InternalTimer( Mobile from, CraftSystem craftSystem, CraftItem craftItem, int iCountMax ) : base( TimeSpan.Zero, TimeSpan.FromSeconds( craftSystem.Delay ), iCountMax )
			{
				m_From = from;
				m_CraftItem = craftItem;
				m_iCount = 0;
				m_iCountMax = iCountMax;
				m_CraftSystem = craftSystem;
			}

			protected override void OnTick()
			{
				m_iCount++;

				if ( m_iCount < m_iCountMax )
				{
					m_CraftItem.PlayCraftEffect( m_From, m_CraftSystem );
				}
				else
				{
					m_From.EndAction( typeof( CraftSystem ) );

					if ( m_CraftItem.SkillCheck( m_From ) )
					{
						m_CraftSystem.PlayEndingEffect( m_From, false, true );

						Item item = Activator.CreateInstance( m_CraftItem.ItemType ) as Item;

						if ( item != null )
						{
							m_From.AddToBackpack( item );
							m_From.PlaySound( 0x57 );
						}
					}
					else
					{
						// SkillCheck failed.
						m_CraftSystem.PlayEndingEffect( m_From, true, m_CraftItem.GiveBackMaterials( m_From ) );
					}
				}
			}
		}
	}
}