// Lumberjacking Definition / Harvest System Script v.0.2 by SickLab

using System;
using Server.Scripts.Items;

namespace Server.Scripts.HarvestSystems
{
	public class DefLumberjacking : HarvestSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Lumberjacking; }
		}

		private static HarvestSystem m_HarvestSystem;

		public static HarvestSystem HarvestSystem
		{
			get
			{
				if ( m_HarvestSystem == null )
					m_HarvestSystem = new DefLumberjacking();

				return m_HarvestSystem;
			}
		}

		private DefLumberjacking() : base( 3, 4, 1.6 )
		{
		}

		public override bool CanHarvest( Mobile from, Item item, HarvestItem harvestItem )
		{
			if ( from.Mounted )
			{
				from.SendLocalizedMessage( 1040016 ); // You cannot use this while riding a mount
				return false;
			}

			if ( from.FindItemOnLayer( Layer.TwoHanded ) != item )
			{
				from.SendLocalizedMessage( 500487 ); // The axe must be equipped for any serious wood chopping
				return false;
			}

			if ( harvestItem == null )
			{
				from.SendLocalizedMessage( 500489 ); // You can't use an axe on that
				return false;
			}

			return true;
		}

		public override void PlayHarvestEffect( Mobile from )
		{
			if ( from.Body.Type == BodyType.Human && !from.Mounted )
				from.Animate( 13, 5, 1, true, false, 0 );

			new InternalTimer( from ).Start();
		}

		// Delay to synchronize the sound with the hit on the tree
		private class InternalTimer : Timer
		{
			private Mobile m_From;

			public InternalTimer( Mobile from ) : base( TimeSpan.FromSeconds( 0.9 ) )
			{
				m_From = from;
			}

			protected override void OnTick()
			{
				m_From.PlaySound( 0x13E );
			}
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool toBackpack, bool noMoreRes, bool hasMovedAway )
		{
			if ( hasMovedAway )
			{
				from.SendMessage( "You have moved too far away to continue hacking" );
				return;
			}

			// Someone has gotten the ressource before you
			if ( noMoreRes )
			{
				from.SendLocalizedMessage( 500495 ); // You hack at the tree for a while, but failed to produce any useable wood
				return;
			}
			
			if ( failed )
			{
				from.SendLocalizedMessage( 500495 ); // You hack at the tree for a while, but failed to produce any useable wood
			}
			else
			{
				if ( toBackpack )
				{
					from.SendLocalizedMessage( 500498 ); // You put some logs into your backpack
				}
				else
				{
					from.SendLocalizedMessage( 500497 ); // You can't place any wood into your backpack

				}
			}
		}

		public override void InitHarvestList()
		{
			/*
			 * ItemID		: Item we can harvest from
			 * MinTotalRes	: Minimum amount of ressource the item can have
			 * MaxTotalRes	: Maximum amount of ressource the item can have
			 * MinSkill		: Minimum skill value you need to be able to harvest
			 * MaxSkill		: Maximum skill value to harvest
			 * TypeRes		: Type of the ressource you will get to harvest this item
			 * MinAmount	: Minimum amount of ressource you can get from this item in one hit
			 * MaxAmount	: Maximum amount of ressource you can get from this item in one hit
			 * RegenDelay	: Delay for the item to regenarate when the item has no more ressource
			 * Message		: The message the mobile will receive when no more ressource available
			 */
			

			int[] m_TreeIDs = new int[]
				{
					0x0CD0, 0x0CD3, 0x0CD6,
					0x0CD8, 0x0CDA, 0x0CDD,
					0x0CE0, 0x0CE3, 0x0CE6,
					0x0CCA, 0x0CCB, 0x0CCC,
					0x0CCD, 0x0C12, 0x0CB8,
					0x0CB9, 0x0CBA, 0x0CBB
				};

			
			AddHarvest( m_TreeIDs, 7, 13, 0, 100, typeof( Log ), 2, 5, TimeSpan.FromMinutes( 15 ), 500488 );
		}
	}
}