// Harvest System Script v.0.2 by SickLab

using System;
using Server.Targeting;

namespace Server.Scripts.HarvestSystems
{
	public class HarvestItem
	{
		private int m_ItemID;
		private int m_MinTotalRes;
		private int m_MaxTotalRes;
		private SkillName m_SkillToMake;
		private double m_MinSkill;
		private double m_MaxSkill;
		private Type m_TypeRes;
		private int m_MinAmount;
		private int m_MaxAmount;
		private TimeSpan m_RegenDelay;
		private int m_LocalizedMessage;

		public HarvestItem( int itemID, int minTotalRes, int maxTotalRes, SkillName skillName, double minSkill, double maxSkill, Type typeRes, int minAmount, int maxAmount, TimeSpan regenDelay, int localizedMessage )
		{
			m_ItemID = itemID;
			m_MinTotalRes = minTotalRes;
			m_MaxTotalRes = maxTotalRes;
			m_SkillToMake = skillName;
			m_MinSkill = minSkill;
			m_MaxSkill = maxSkill;
			m_TypeRes = typeRes;
			m_MinAmount = minAmount;
			m_MaxAmount = maxAmount;
			m_RegenDelay = regenDelay;
			m_LocalizedMessage = localizedMessage;
		}

		public int ItemID { get { return m_ItemID; } }
		public Type TypeRes { get { return m_TypeRes; } }
		public int MinTotalRes { get { return m_MinTotalRes; } }
		public int MaxTotalRes { get { return m_MaxTotalRes; } }
		public int MinAmount { get { return m_MinAmount; } }
		public int MaxAmount { get { return m_MaxAmount; } }
		public SkillName SkillToMake { get { return m_SkillToMake; } }
		public double MinSkill{ get { return m_MinSkill; } }
		public double MaxSkill { get { return m_MaxSkill;} }
		public TimeSpan RegenDelay { get { return m_RegenDelay;} }

		public void Harvest( Mobile from, int ItemID, int X, int Y, int Z, HarvestSystem harvestSystem )
		{
			HarvestUsed harvestUsed = harvestSystem.SearchFor( from, ItemID, X, Y, Z, this );
			if ( harvestUsed.Amount > 0 )
			{
				if ( from.BeginAction( typeof( HarvestSystem ) ) )
				{
					int iMin = harvestSystem.MinHarvestEffect;
					int iMax = ( harvestSystem.MaxHarvestEffect - iMin ) + 1;
					int iRandom = Utility.Random( iMax );
					iRandom += iMin + 1;
					new InternalTimer( from, harvestSystem, this, harvestUsed, iRandom ).Start();
				}
				else
				{
					from.SendLocalizedMessage( 500119 ); // You must wait to perform another action
				}
			}
			else
			{
				from.SendLocalizedMessage( m_LocalizedMessage );
			}
		}

		public bool SkillCheck( Mobile from, HarvestUsed harvestUsed, HarvestSystem harvestSystem, ref Type ressource, ref int amount )
		{
			int iMin = this.MinAmount;
			int iMax = ( this.MaxAmount - iMin ) + 1;
			amount = Utility.Random( iMax ) + iMin;

			// If this region has special resource to harvest
			if ( harvestUsed.SpecialRes )
			{
				HarvestResDef harvestResDef = harvestSystem.HarvestResDefs.SearchFor( harvestUsed.SpecialResource );

				// Is it defined with AddResourceInfo function?
				if ( harvestResDef != null )
				{
					if ( from.CheckSkill( m_SkillToMake, harvestResDef.MinSkill, harvestResDef.MaxSkill ) )
					{
						// A Special Resource has been founded, initialize it
						if ( !harvestUsed.SpecialFounded )
						{
							int iMinRes = harvestResDef.MinTotalRes;
							int iMaxRes = ( harvestResDef.MaxTotalRes - iMinRes ) + 1;
							harvestUsed.Amount = Utility.Random( iMaxRes ) + iMinRes;
						
							harvestUsed.SpecialFounded = true;
						}
						ressource = harvestUsed.SpecialResource;
						return true;
					}
				}
			}

			if ( harvestUsed.SpecialFounded )
			{
				// Since the special resource has been found, and the player failed
				// the special resource check, return to avoid mining default resource.
				return false;
			}

			// Yes, two skillcheck will be done if the one for special resource failed.
			if ( from.CheckSkill( m_SkillToMake, m_MinSkill, m_MaxSkill ) )
			{
				ressource = harvestUsed.DefaultResource;
				return true;
			}

			return false;
		}

		private class InternalTimer : Timer
		{
			private Mobile m_From;
			private int m_iCount;
			private int m_iCountMax;
			private HarvestItem m_HarvestItem;
			private HarvestSystem m_HarvestSystem;
			private HarvestUsed m_HarvestUsed;

			public InternalTimer( Mobile from, HarvestSystem harvestSystem, HarvestItem harvestItem, HarvestUsed harvestUsed, int iCountMax ) : base( TimeSpan.FromSeconds( 0 ), TimeSpan.FromSeconds( harvestSystem.Delay ), iCountMax )
			{
				m_From = from;
				m_HarvestItem = harvestItem;
				m_HarvestSystem = harvestSystem;
				m_HarvestUsed = harvestUsed;
				m_iCount = 0;
				m_iCountMax = iCountMax;
			}

			protected override void OnTick()
			{
				if ( !m_From.InRange( m_HarvestUsed, 1 ) )
				{
					Stop();
					m_From.EndAction( typeof( HarvestSystem ) );
					m_HarvestSystem.PlayEndingEffect( m_From, false, false, false, true );
					return;
				}
				
				m_From.Direction = m_From.GetDirectionTo( m_HarvestUsed );

				m_iCount++;

				if ( m_iCount < m_iCountMax )
				{
					m_HarvestSystem.PlayHarvestEffect( m_From );
				}
				else
				{
					m_From.EndAction( typeof( HarvestSystem ) );

					int Amount = 0;
					Type Ressource = null;

					if ( m_HarvestItem.SkillCheck( m_From, m_HarvestUsed, m_HarvestSystem, ref Ressource, ref Amount ) )
					{
						if ( Amount > m_HarvestUsed.Amount )
							Amount = m_HarvestUsed.Amount;

						// if Amount = 0 here, it means that two mobile were harvesting the same item, and someone got
						// the ressource before the actual mobile.
						if ( Amount <= 0 )
						{
							m_HarvestSystem.PlayEndingEffect( m_From, false, false, true, false );
                            return;
						}

						Item item = Activator.CreateInstance( Ressource ) as Item;

						m_HarvestUsed.Amount -= Amount;
						item.Amount = Amount;

						bool ToBackpack = false;
						if ( m_From.AddToBackpack( item ) )
						{
							m_From.PlaySound(0x57);
							ToBackpack = true;
						}

						m_HarvestSystem.PlayEndingEffect( m_From, false, ToBackpack, false, false );

					}
					else
					{
						// SkillCheck failed.
						m_HarvestSystem.PlayEndingEffect( m_From, true, false, false, false );
					}
				}
			}
		}
	}
}
