// Harvest System Script v.0.2 by SickLab

using System;

namespace Server.Scripts.HarvestSystems
{
	public class HarvestUsed : IPoint2D, IPoint3D
	{
		private int m_ItemID;
		private int m_X;
		private int m_Y;
		private int m_Z;
		private int m_Amount;
		private HarvestSystem m_HarvestSystem;
		private TimeSpan m_RegenDelay;
		private Type m_DefaultResType;
		private Type m_SpecialResType;
		private bool m_SpecialFounded;
		private bool m_SpecialRes;

		public HarvestUsed( int itemID, int x, int y, int z, int amount, HarvestSystem harvestSystem, TimeSpan regenDelay, Type defaultResType, Type specialResType )
		{
			m_ItemID = itemID;
			m_X = x;
			m_Y = y;
			m_Z = z;
			m_Amount = amount;
			m_HarvestSystem = harvestSystem;
			m_RegenDelay = regenDelay;
			m_DefaultResType = defaultResType;
			m_SpecialResType = specialResType;
			m_SpecialFounded = false;

			if ( m_SpecialResType == m_DefaultResType )
				m_SpecialRes = false;
			else
				m_SpecialRes = true;
		}

		public void Remove()
		{
			m_HarvestSystem.RemoveHarvestUsed( this );
		}

		public Type DefaultResource { get { return m_DefaultResType; } }
		public Type SpecialResource { get { return m_SpecialResType; } }
		public bool SpecialFounded { get { return m_SpecialFounded; } set { m_SpecialFounded = value; } }
		public bool SpecialRes { get { return m_SpecialRes; } }
		public int ItemID { get { return m_ItemID; } }
		public int X { get { return m_X; } }
		public int Y { get { return m_Y; } }
		public int Z { get { return m_Z; } }
		public HarvestSystem HarvestSystem { get { return m_HarvestSystem; } }
		public TimeSpan RegenDelay { get { return m_RegenDelay; } }
		
		public int Amount
		{
			get
			{
				return m_Amount;
			}
			set
			{
				m_Amount = value;
				if ( m_Amount <= 0 )
				{
					new InternalTimer( this ).Start();
				}
			}
		}

		private class InternalTimer : Timer
		{
			private HarvestUsed m_HarvestUsed;

			public InternalTimer( HarvestUsed harvestUsed ) : base( harvestUsed.RegenDelay )
			{
				m_HarvestUsed = harvestUsed;
			}

			protected override void OnTick()
			{
				m_HarvestUsed.Remove();
			}
		}
	}
}
