using System;

namespace Server.Items
{
	public enum TrapType
	{
		None,
		MagicTrap
	}

	public abstract class TrapableContainer : Container
	{
		private TrapType m_TrapType;
		private int m_TrapPower;

		[CommandProperty( AccessLevel.GameMaster )]
		public TrapType TrapType
		{
			get
			{
				return m_TrapType;
			}
			set
			{
				m_TrapType = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int TrapPower
		{
			get
			{
				return m_TrapPower;
			}
			set
			{
				m_TrapPower = value;
			}
		}

		public TrapableContainer( int itemID, int gumpID, int defaultDropSound, Rectangle2D bounds ) : base( itemID, gumpID, defaultDropSound, bounds )
		{
		}

		public TrapableContainer( Serial serial ) : base( serial )
		{
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( m_TrapType != TrapType.None )
			{
				//from.SendLocalizedMessage( 1005517 ); // You fail to notice the trap

				switch ( m_TrapType )
				{
					case TrapType.MagicTrap:
					{
						this.LabelTo( from, "Damage: {0}", m_TrapPower );

						from.Damage( m_TrapPower );

						Point3D loc = GetWorldLocation();

						Effects.PlaySound( loc, Map, 0x207 );

						for ( int x = -1; x <= 1; ++x )
							for ( int y = -1; y <= 1; ++y )
								Effects.SendLocationEffect( new Point3D( loc.X + x, loc.Y + y, loc.Z ), Map, 0x36BD, 20 );

						break;
					}
				}
			
				m_TrapType = TrapType.None;
			}
			else
			{
				base.OnDoubleClick( from );
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( (int) m_TrapPower );
			writer.Write( (int) m_TrapType );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_TrapPower = reader.ReadInt();

					goto case 0;
				}

				case 0:
				{
					m_TrapType = (TrapType)reader.ReadInt();

					break;
				}

			}
		}
	}
}