using System;
using Server.Targeting;
using Server.Items;
using Server.Scripts.Items;

namespace Server.Items
{
	public class FishingPole : Item
	{
		[Constructable]
		public FishingPole() : base( 0x0DC0 )
		{
			Layer = Layer.OneHanded;
		}

		public FishingPole( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( from.Mounted )
			{
				from.SendLocalizedMessage( 500971 ); // You can't fish while riding!
			}
			else
			{
				from.SendLocalizedMessage( 500974 ); // What water do you want to fish in?
				from.Target = new InternalTarget();
			}
		}

		private class InternalTarget : Target
		{
			private static int[] m_WaterIDs = new int[]
				{
					0x00A8, 0x00AB,
					0x0136, 0x0137,
					0x5797, 0x579C,
					0x746E, 0x7485,
					0x7490, 0x74AB,
					0x74B5, 0x75D5
				};

			public InternalTarget() : base( 6, true, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object o )
			{
				int itemID;

				if ( o is StaticTarget )
				{
					itemID = ((StaticTarget)o).ItemID | 0x4000;
				}
				else if ( o is LandTarget )
				{
					itemID = from.Map.Tiles.GetLandTile( ((LandTarget)o).X, ((LandTarget)o).Y ).ID & 0x3FFF;
				}
				else
				{
					return;
				}

				IPoint3D p = o as IPoint3D;

				if ( p == null )
					return;

				for ( int i = 0; i < m_WaterIDs.Length; i += 2 )
				{
					if ( itemID >= m_WaterIDs[i] && itemID <= m_WaterIDs[i + 1] )
					{
						from.RevealingAction();

						if ( from.Mounted )
						{
							from.SendLocalizedMessage( 500971 ); // You can't fish while riding!
						}
						else if ( from.BeginAction( typeof( FishingPole ) ) )
						{
							Effects.SendLocationEffect( p, from.Map, 0x352D, 4 );
							Effects.PlaySound( p, from.Map, 0x23F );

							new FishingTimer( from, p ).Start();
						}
						else
						{
							from.SendLocalizedMessage( 500972 ); // You are already fishing.
						}

						return;
					}
				}
			}
		}

		private class FishingTimer : Timer
		{
			private Mobile m_Mobile;
			private IPoint3D m_Point;

			public FishingTimer( Mobile from, IPoint3D p ) : base( TimeSpan.FromSeconds( 5.0 ) )
			{
				m_Mobile = from;
				m_Point = p;
			}

			protected override void OnTick()
			{
				Mobile from = m_Mobile;

				from.EndAction( typeof( FishingPole ) );

				if ( from.Mounted )
				{
					from.SendLocalizedMessage( 500971 ); // You can't fish while riding!
					return;
				}
				else if ( !from.InRange( m_Point, 6 ) )
				{
					from.SendLocalizedMessage( 500976 ); // You need to be closer to the water to fish!
				}
				else
				{
					from.RevealingAction();

					if ( !from.CheckSkill( SkillName.Fishing, 0.0, 100.0 ) )
					{
						from.SendLocalizedMessage( 503171 ); // You fish a while, but fail to catch anything.
					}
					else
					{
						double specialChance = (from.Skills[SkillName.Fishing].Value - 80.0) / 10.0;
						string itemName = null;

						if ( specialChance > Utility.RandomDouble() * 100.0 )
						{
							int random = Utility.Random( 50 );

							if ( random == 0 )
							{
								itemName = "#1041080"; // a message in a bottle
								from.AddToBackpack( new MessageInABottle() );
							}
							else if ( random <= 10 )
							{
								itemName = "#1041511"; // a tattered, plainly drawn treasure map
							}
							else if ( random <= 20 )
							{
								from.SendLocalizedMessage( 1042635 ); // Your fishing pole bends as you pull a big fish from the depths!
								from.AddToBackpack( new BigFish() );
							}
							else if ( random <= 30 )
							{
								itemName = "#1041079"; // a special fishing net
								from.AddToBackpack( new SpecialFishingNet() );
							}
							else
							{
								itemName = "a fish";
								from.AddToBackpack( new Fish() );
							}
						}
						else
						{
							int random = Utility.Random( 500 );

							if ( random == 0 )
							{
								itemName = "magical fish";
								// TODO: Magical fish
							}
							else if ( random <= 100 )
							{
								itemName = "old shoes";

								switch ( Utility.Random( 4 ) )
								{
									case 0: from.AddToBackpack( new Shoes( Utility.RandomYellowHue() ) ); break;
									case 1: from.AddToBackpack( new Boots( Utility.RandomYellowHue() ) ); break;
									case 2: from.AddToBackpack( new ThighBoots( Utility.RandomYellowHue() ) ); break;
									case 3: from.AddToBackpack( new Sandals() ); break;
								}
							}
							else if ( random <= 200 )
							{
								itemName = "a fish";
								from.AddToBackpack( new Fish() );
							}
							else
							{
								from.SendLocalizedMessage( 503171 ); // You fish a while, but fail to catch anything.
							}
						}

						if ( itemName != null )
							from.SendLocalizedMessage( 1043297, itemName );
					}
				}
			}
		}
	}
}