using System;
using Server.Network;
using Server.Prompts;

namespace Server.Items
{
	public class RecallRune : Item
	{
		private string m_Description;
		private bool m_Marked;
		private Point3D m_Target;
		private Map m_TargetMap;

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( (string) m_Description );
			writer.Write( (bool) m_Marked );
			writer.Write( (Point3D) m_Target );
			writer.Write( (Map) m_TargetMap );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Description = reader.ReadString();
					m_Marked = reader.ReadBool();
					m_Target = reader.ReadPoint3D();
					m_TargetMap = reader.ReadMap();

					break;
				}
			}
		}

		[CommandProperty( AccessLevel.Counselor, AccessLevel.GameMaster )]
		public string Description
		{
			get
			{
				return m_Description;
			}
			set
			{
				m_Description = value;
			}
		}

		[CommandProperty( AccessLevel.Counselor, AccessLevel.GameMaster )]
		public bool Marked
		{
			get
			{
				return m_Marked;
			}
			set
			{
				if ( m_Marked != value )
				{
					m_Marked = value;
					CalculateHue();
				}
			}
		}

		[CommandProperty( AccessLevel.Counselor, AccessLevel.GameMaster )]
		public Point3D Target
		{
			get
			{
				return m_Target;
			}
			set
			{
				m_Target = value;
			}
		}

		[CommandProperty( AccessLevel.Counselor, AccessLevel.GameMaster )]
		public Map TargetMap
		{
			get
			{
				return m_TargetMap;
			}
			set
			{
				if ( m_TargetMap != value )
				{
					m_TargetMap = value;
					CalculateHue();
				}
			}
		}

		private void CalculateHue()
		{
			if ( !m_Marked )
			{
				Hue = 0;
			}
			else if ( m_TargetMap == Map.Trammel )
			{
				Hue = 2120;
			}
			else if ( m_TargetMap == Map.Felucca )
			{
				Hue = 2118;
			}
			else if ( m_TargetMap == Map.Ilshenar )
			{
				Hue = 2401;
			}
		}

		public void Mark( Mobile m )
		{
			m_Marked = true;
			m_Target = m.Location;
			m_TargetMap = m.Map;
			CalculateHue();
		}

		public override void OnSingleClick( Mobile from )
		{
			base.OnSingleClick( from);

			NetState ns = from.NetState;

			if ( ns != null )
			{
				string text;

				if ( m_Marked )
				{
					if ( (text = m_Description) == null || (text = text.Trim()).Length <= 0 )
					{
						text = "(indescript)";
					}
				}
				else
				{
					text = "(unmarked)";
				}

				ns.Send( new UnicodeMessage( Serial, ItemID, MessageType.Label, 0x3B2, 3, "ENU", "", text ) );
			}
		}

		public override void OnDoubleClick( Mobile from )
		{
			int number;

			if ( m_Marked )
			{
				number = 501804; // Please enter a description for this marked object.

				from.Prompt = new RenamePrompt( this );
			}
			else
			{
				number = 501805; // That rune is not yet marked.
			}

			from.SendLocalizedMessage( number );
		}

		private class RenamePrompt : Prompt
		{
			private RecallRune m_Rune;

			public RenamePrompt( RecallRune rune )
			{
				m_Rune = rune;
			}

			public override void OnResponse( Mobile from, string text )
			{
				m_Rune.Description = text;
			}
		}

		[Constructable]
		public RecallRune() : base( 0x1F14 )
		{
			Weight = 1.0;
			CalculateHue();
		}

		public RecallRune( Serial serial ) : base( serial )
		{
		}
	}
}