using System;
using System.Collections;
using Server.Scripts.Misc;
using Server.Items;
using Server.Scripts.Items;
using Server.Scripts.Mobiles.Animals.Mounts;
using Server.Scripts.Items.Armor.Studded;
using Server.Scripts.Items.Weapons.Ranged;

namespace Server.Scripts.Mobiles.People
{
	public class ArcherGuard : BaseGuard
	{
		private Timer m_AttackTimer, m_IdleTimer;

		private Mobile m_Focus;

		public ArcherGuard( Mobile target ) : base( target )
		{
			InitStats( 100, 125, 25 );
			Title = "the guard";

			Hue = Utility.RandomSkinHue();

			if ( Female = Utility.RandomBool() )
			{
				Body = 0x191;
				Name = NameList.RandomFemaleName();
			}
			else
			{
				Body = 0x190;
				Name = NameList.RandomMaleName();
			}

			new GrayHorse().Rider = this;

			AddItem( new StuddedChest() );
			AddItem( new StuddedArms() );
			AddItem( new StuddedGloves() );
			AddItem( new StuddedGorget() );
			AddItem( new StuddedLegs() );
			AddItem( new Boots() );
			AddItem( new SkullCap() );

			Bow bow = new Bow();

			bow.Movable = false;
			bow.Newbied = true;
			bow.Crafter = this;
			bow.Quality = WeaponQuality.Exceptional;

			AddItem( bow );

			Container pack = new Backpack();

			pack.Movable = false;
			pack.Newbied = true;

			Arrow arrows = new Arrow( 250 );

			arrows.Newbied = true;

			pack.DropItem( arrows );
			pack.DropItem( new Gold( 250, 500 ) );

			AddItem( pack );

			Skills[SkillName.Anatomy].Base = 120.0;
			Skills[SkillName.Tactics].Base = 120.0;
			Skills[SkillName.Archery].Base = 120.0;
			Skills[SkillName.MagicResist].Base = 120.0;

			this.NextCombatTime = DateTime.Now + TimeSpan.FromSeconds( 0.5 );
			this.Focus = target;
		}

		public ArcherGuard( Serial serial ) : base( serial )
		{
		}

		public override bool OnBeforeDeath()
		{
			if ( m_Focus != null && m_Focus.Alive )
				new AvengeTimer( m_Focus ).Start(); // If a guard dies, three more guards will spawn

			return base.OnBeforeDeath();
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public override Mobile Focus
		{
			get
			{
				return m_Focus;
			}
			set
			{
				if ( m_Focus != value )
				{
					if ( value != null )
						this.AggressiveAction( value );

					Combatant = value;

					if ( value != null )
					{
						Say( 500131 ); // Thou wilt regret thine actions, swine!
					}
					else if ( m_Focus != null )
					{
						Say( "Thou hast suffered thy punishment, scoundrel." );
					}

					m_Focus = value;

					if ( m_AttackTimer != null )
					{
						m_AttackTimer.Stop();
						m_AttackTimer = null;
					}

					if ( m_IdleTimer != null )
					{
						m_IdleTimer.Stop();
						m_IdleTimer = null;
					}

					if ( m_Focus != null )
					{
						m_AttackTimer = new AttackTimer( this );
						m_AttackTimer.Start();
					}
					else
					{
						m_IdleTimer = new IdleTimer( this );
						m_IdleTimer.Start();
					}
				}
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_Focus );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Focus = reader.ReadMobile();

					if ( m_Focus != null )
					{
						m_AttackTimer = new AttackTimer( this );
						m_AttackTimer.Start();
					}
					else
					{
						m_IdleTimer = new IdleTimer( this );
						m_IdleTimer.Start();
					}

					break;
				}
			}
		}

		public override void OnAfterDelete()
		{
			if ( m_AttackTimer != null )
			{
				m_AttackTimer.Stop();
				m_AttackTimer = null;
			}

			if ( m_IdleTimer != null )
			{
				m_IdleTimer.Stop();
				m_IdleTimer = null;
			}

			base.OnAfterDelete();
		}

		private class AvengeTimer : Timer
		{
			private Mobile m_Focus;

			public AvengeTimer( Mobile focus ) : base( TimeSpan.FromSeconds( 2.5 ), TimeSpan.FromSeconds( 1.0 ), 3 ) // After 2.5 seconds, one guard will spawn every 1.0 second, three times
			{
				m_Focus = focus;
			}

			protected override void OnTick()
			{
				BaseGuard.Spawn( m_Focus, 1, true );
			}
		}

		private class AttackTimer : Timer
		{
			private ArcherGuard m_Owner;
			private bool m_Shooting;

			public AttackTimer( ArcherGuard owner ) : base( TimeSpan.FromSeconds( 0.25 ), TimeSpan.FromSeconds( 0.1 ) )
			{
				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Deleted )
				{
					Stop();
					return;
				}

				// <?>
				m_Owner.Criminal = false;
				m_Owner.Kills = 0;
				// </?>

				Mobile target = m_Owner.Combatant;

				if ( target == null )
				{
					m_Owner.Focus = null;
				}
				else if ( !m_Owner.InRange( target, 20 ) )
				{
					m_Owner.Focus = null;
				}
				/*else if ( !m_Owner.InRange( target, 10 ) )
				{
					Effects.SendLocationEffect( m_Owner.Location, m_Owner.Map, 0x3728, 13 );

					m_Owner.Location = target.Location;

					Effects.SendLocationEffect( m_Owner.Location, m_Owner.Map, 0x3728, 13 );
					m_Owner.PlaySound( 0x1FE );
				}*/
				else
				{
					if ( m_Shooting && (TimeToSpare() || OutOfMaxDistance( target )) )
						m_Shooting = false;
					else if ( !m_Shooting && InMinDistance( target ) )
						m_Shooting = true;

					if ( !m_Shooting )
					{
						if ( m_Owner.InRange( target, 1 ) )
						{
							m_Owner.Move( (Direction)(m_Owner.GetDirectionTo( target ) - 4) | Direction.Running ); // Too close, move away
						}
						else if ( !m_Owner.InRange( target, 2 ) )
						{
							m_Owner.Move( m_Owner.GetDirectionTo( target ) | Direction.Running );
						}
					}
				}
			}

			private bool TimeToSpare()
			{
				return (m_Owner.NextCombatTime - DateTime.Now) > TimeSpan.FromSeconds( 1.0 );
			}

			private bool OutOfMaxDistance( Mobile target )
			{
				return !m_Owner.InRange( target, m_Owner.Weapon.MaxRange );
			}

			private bool InMinDistance( Mobile target )
			{
				return m_Owner.InRange( target, 4 );
			}
		}

		private class IdleTimer : Timer
		{
			private ArcherGuard m_Owner;
			private int m_Stage;

			public IdleTimer( ArcherGuard owner ) : base( TimeSpan.FromSeconds( 2.0 ), TimeSpan.FromSeconds( 2.5 ) )
			{
				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Deleted )
				{
					Stop();
					return;
				}

				Mobile target = m_Owner.Combatant;

				if ( target == null )
				{
					if ( (m_Stage++ % 4) == 0 || !m_Owner.Move( m_Owner.Direction ) )
						m_Owner.Direction = (Direction)Utility.Random( 8 );

					if ( m_Stage > 16 )
					{
						Effects.SendLocationEffect( m_Owner.Location, m_Owner.Map, 0x3728, 13 );
						m_Owner.PlaySound( 0x1FE );

						m_Owner.Delete();
					}
				}
				else
				{
					m_Owner.Focus = target;
				}
			}
		}
	}
}