using System;
using System.Collections;
using Server.Scripts.Misc;
using Server.Items;
using Server.Scripts.Items.Weapons.Swords;

namespace Server.Scripts.Mobiles.Monsters.Melee
{
	[Dispellable]
	public class BladeSpirits : AggressiveAI
	{
		private Timer m_Timer;

		[Constructable]
		public BladeSpirits()
		{
			this.Body = 574;
			this.Kills = 500; // Murderer
			this.Name = "blade spirits";

			// TODO: 1200 hits max, 250 stamina
			this.InitStats( 150, 150, 100 );

			this.Skills[SkillName.Swords].Base = 91.0;
			this.Skills[SkillName.Tactics].Base = 90.0;
			this.Skills[SkillName.MagicResist].Base = 70.0;

			m_Timer = new ExpireTimer( this );
			m_Timer.Start();

			Katana kat = new Katana();

			kat.Newbied = true;
			kat.Movable = false;
			kat.Quality = WeaponQuality.Exceptional;

			this.AddItem( kat );
		}

		public BladeSpirits( Serial serial ) : base( serial )
		{
			m_Timer = new ExpireTimer( this );
			m_Timer.Start();
		}

		public override void OnAfterDelete()
		{
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		private class ExpireTimer : Timer
		{
			private BladeSpirits m_Owner;

			public ExpireTimer( BladeSpirits owner ) : base( TimeSpan.FromMinutes( 1.5 ) )
			{
				this.Priority = TimerPriority.FiveSeconds;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				m_Owner.Delete();
			}
		}
	}
}