// Hair Dye Bottle v1.0
// Scripted by Kiwi
// 10/06/2002
// http://www.playuo.net


using System;
using System.Text;
using Server.Gumps;
using Server.Network;

namespace Server.Scripts.Items
{
	public class HairDye : Item
	{
		[Constructable]
		public HairDye() : base( 0xEFF )
		{
			Name = "hair dye";
			
		}

		public HairDye( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnDoubleClick( Mobile from )
		{
			from.CloseGump( typeof( HairDyeGump ) );
			from.SendGump( new HairDyeGump( this ) );
		}
	}

	public class HairDyeGump : Gump
	{
		private HairDye m_HairDye;

		private class HairDyeEntry
		{
			private string m_Name;
			private int m_HueStart;
			private int m_HueCount;

			public string Name
			{
				get
				{
					return m_Name;
				}
			}

			public int HueStart
			{
				get
				{
					return m_HueStart;
				}
			}

			public int HueCount
			{
				get
				{
					return m_HueCount;
				}
			}

			public HairDyeEntry( string name, int hueStart, int hueCount )
			{
				m_Name = name;
				m_HueStart = hueStart;
				m_HueCount = hueCount;
			}
		}

		private static HairDyeEntry[] m_Entries = new HairDyeEntry[]
			{
				new HairDyeEntry( "Browns", 1601, 27 ),
				new HairDyeEntry( "Chesnut", 1628, 27 ),
				new HairDyeEntry( "Auburn", 1501, 32 ),
				new HairDyeEntry( "Blues", 1301, 32 ),
				new HairDyeEntry( "Greens", 1401, 32 ),
				new HairDyeEntry( "Reds", 1201, 24 ),
				new HairDyeEntry( "Lights", 2401, 30 ),
				new HairDyeEntry( "Blondes", 2213, 6 ),
				new HairDyeEntry( "Blacks", 1102, 8 ),
				new HairDyeEntry( "Beiges", 1110, 8 ),
				new HairDyeEntry( "Golden", 1118, 16 ),
				new HairDyeEntry( "Dark Brown", 1134, 16 )
			};

		public HairDyeGump( HairDye dye ) : base( 0, 0 )
		{
			m_HairDye = dye;

			AddPage( 0 );
			AddBackground( 100, 10, 350, 370, 2600 );
			AddBackground( 120, 54, 110, 270, 5100 );
			AddHtmlLocalized( 180, 25, 200, 20, 1011013, false, false );		// Hair Color Selection Menu
			AddHtmlLocalized( 150, 330, 150, 20, 1011014, false, false );		// Dye my hair this color!
			AddButton( 375, 330, 0xFA5, 0xFA7, 1, GumpButtonType.Reply, 0 );    // DYE HAIR

			for ( int i = 0; i < m_Entries.Length; ++i )
			{
				AddLabel( 130, 59 + (i * 22), m_Entries[i].HueStart - 1, m_Entries[i].Name );
				AddButton( 207, 60 + (i * 22), 5224, 5224, 0, GumpButtonType.Page, i + 1 );
			}

			for ( int i = 0; i < m_Entries.Length; ++i )
			{
				HairDyeEntry e = m_Entries[i];

				AddPage( i + 1 );

				for ( int j = 0; j < e.HueCount; ++j )
				{
					AddLabel( 278 + ((j / 16) * 80), 52 + ((j % 16) * 17), e.HueStart + j - 1, "*****" );
					AddRadio( 260 + ((j / 16) * 80), 52 + ((j % 16) * 17), 208, 209, false, (i * 100) + j );
				}
			}
		}

		public override void OnResponse( NetState from, RelayInfo info )
		{
			if ( m_HairDye.Deleted )
				return;

			Mobile m = from.Mobile;
			int[] switches = info.Switches;

			if ( info.ButtonID != 0 && switches.Length > 0 )
			{
				Item hair = m.Hair;
				Item beard = m.Beard;

				if ( hair == null && beard == null )
				{
					m.SendLocalizedMessage( 502623 );	// You have no hair to dye and cannot use this
				}
				else
				{
					// To prevent this from being exploited, the hue is abstracted into an internal list

					int entryIndex = switches[0] / 100;
					int hueOffset = switches[0] % 100;

					if ( entryIndex >= 0 && entryIndex < m_Entries.Length )
					{
						HairDyeEntry e = m_Entries[entryIndex];

						if ( hueOffset >= 0 && hueOffset < e.HueCount )
						{
							int hue = e.HueStart + hueOffset;

							if ( hair != null )
								hair.Hue = hue;

							if ( beard != null )
								beard.Hue = hue;

							m.SendLocalizedMessage( 501199 );  // You dye your hair
							m_HairDye.Delete();
						}
					}
				}
			}
			else
			{
				m.SendLocalizedMessage( 501200 ); // You decide not to dye your hair
			}
		}
	}
}